@@ -17,7 +17,7 @@ use Text::Abbrev();
 use Exporter ();
 our @ISA = qw{ Exporter };
 
-our $VERSION = '0.000_014';
+our $VERSION = '0.000_015';
 
 our @EXPORT_OK = qw{
     __am_or_pm
@@ -177,6 +177,15 @@ sub _fmt_cond {
 }
 
 {
+    # NOTE - I _was_ using assignment to $_[2] followed by a goto to
+    # dispatch _fmt_number__2() and _fmt_number_02(). But this produced
+    # test failures under 5.8.5, which I was able to reproduce, though
+    # not under -d:ptkdb, which suggests it was an optimizer problem.
+    # Only _fmt_number__2() resulted in the failures, but I recoded
+    # both, plus the couple dispatches directly to _fmt_number() since
+    # the previous dispatch scheme for all three involved fiddling with
+    # the contents of @_. There is still a goto inside _fmt_number__2(),
+    # but since I no longer modify @_, I have let that stand.
     my %spec = (
 	A	=> sub { $_[0]->__fmt_shire_traditional() ?
 		    __trad_weekday_name( $_[0]->__fmt_shire_day_of_week() ) :
@@ -189,14 +198,14 @@ sub _fmt_cond {
 	B	=> sub { __month_name( $_[0]->__fmt_shire_month() ) },
 	b	=> sub { __month_abbr( $_[0]->__fmt_shire_month() ) },
 	C	=> sub {
-		    $_[2] = int( $_[0]->__fmt_shire_year() / 100 );
-		    goto &_fmt_number_02;
+		    return _fmt_number_02( @_[ 0, 1 ],
+			int( $_[0]->__fmt_shire_year() / 100 ) );
 		},
 	c	=> sub { __format( $_[0], '%{{%a %x||||%x}} %X' ) },
 	D	=> sub { __format( $_[0], '%{{%m/%d||%Ee}}/%y' ) },
 	d	=> sub {
-		    $_[2] = $_[0]->__fmt_shire_day();
-		    goto &_fmt_number_02;
+		    return _fmt_number_02( @_[ 0, 1 ],
+			$_[0]->__fmt_shire_day() );
 		},
 	Ed	=> \&_fmt_on_date,
 	EE	=> sub { __holiday_name( $_[0]->__fmt_shire_month() ? 0 :
@@ -207,50 +216,74 @@ sub _fmt_cond {
 	Ex	=> sub { __format( $_[0],
 		    '%{{%A %-e %B %Y||%A %EE %Y||%EE %Y}}' ) },
 	e	=> sub {
-		    $_[2] = $_[0]->__fmt_shire_day();
-		    goto &_fmt_number__2;
+		    return _fmt_number__2( @_[ 0, 1 ],
+			$_[0]->__fmt_shire_day() );
 		},
 	F	=> sub { __format( $_[0], '%Y-%{{%m-%d||%Ee}}' ) },
 #	G	Same as Y by definition of Shire calendar
-	H	=> sub { $_[2] = $_[0]->__fmt_shire_hour(); goto &_fmt_number_02 },
+	H	=> sub {
+		    return _fmt_number_02( @_[ 0, 1 ],
+			$_[0]->__fmt_shire_hour() );
+		},
 #	h	Same as b by definition of strftime()
 	I	=> sub {
-		    $_[2] = ( $_[0]->__fmt_shire_hour() || 0 ) % 12 || 12;
-		    goto &_fmt_number_02;
+		    return _fmt_number_02( @_[ 0, 1 ],
+			( $_[0]->__fmt_shire_hour() || 0 ) % 12 || 12,
+		    );
 		},
 	j	=> sub {
 		    defined $_[1]{wid}
 			or $_[1]{wid} = 3;
-		    $_[2] = __date_to_day_of_year( $_[0]->__fmt_shire_year(),
-			$_[0]->__fmt_shire_month(), $_[0]->__fmt_shire_day() );
-		    goto &_fmt_number;
+		    return _fmt_number( @_[ 0, 1 ],
+			__date_to_day_of_year(
+			    $_[0]->__fmt_shire_year(),
+			    $_[0]->__fmt_shire_month(),
+			    $_[0]->__fmt_shire_day(),
+			),
+		    );
+		},
+	k	=> sub {
+		    return _fmt_number__2( @_[ 0, 1 ],
+			$_[0]->__fmt_shire_hour() );
 		},
-	k	=> sub { $_[2] = $_[0]->__fmt_shire_hour(); goto &_fmt_number__2; },
 	l	=> sub {
-		    $_[2] = ( $_[0]->__fmt_shire_hour() || 0 ) % 12 || 12;
-		    goto &_fmt_number__2;
+		    return _fmt_number__2( @_[ 0, 1 ],
+			( $_[0]->__fmt_shire_hour() || 0 ) % 12 || 12 );
+		},
+	M	=> sub {
+		    return _fmt_number_02( @_[ 0, 1 ],
+			$_[0]->__fmt_shire_minute() );
+		},
+	m	=> sub {
+		    return _fmt_number_02( @_[ 0, 1 ],
+			$_[0]->__fmt_shire_month() );
 		},
-	M	=> sub { $_[2] = $_[0]->__fmt_shire_minute(); goto &_fmt_number_02; },
-	m	=> sub { $_[2] = $_[0]->__fmt_shire_month(); goto &_fmt_number_02; },
 	N	=> sub {
 		    defined $_[1]{wid}
 			or $_[1]{wid} = 9;
-		    $_[2] = $_[0]->__fmt_shire_nanosecond();
-		    goto &_fmt_number;
+		    return _fmt_number( @_[ 0, 1 ],
+			$_[0]->__fmt_shire_nanosecond(),
+		    );
 		},
 	n	=> sub { "\n" },
 	P	=> sub { lc __am_or_pm( $_[0]->__fmt_shire_hour() ) },
 	p	=> sub { uc __am_or_pm( $_[0]->__fmt_shire_hour() ) },
 	R	=> sub { __format( $_[0], '%H:%M' ) },
 	r	=> sub { __format( $_[0], '%I:%M:%S %p' ) },
-	S	=> sub { $_[2] = $_[0]->__fmt_shire_second(); goto &_fmt_number_02; },
+	S	=> sub {
+		    return _fmt_number_02( @_[ 0, 1 ],
+			$_[0]->__fmt_shire_second() );
+		},
 	s	=> sub { $_[0]->__fmt_shire_epoch() },
 	T	=> sub { __format( $_[0], '%H:%M:%S' ) },
 	t	=> sub { "\t" },
 	U	=> sub {
-		    $_[2] = __week_of_year(
-			$_[0]->__fmt_shire_month(), $_[0]->__fmt_shire_day() );
-		    goto &_fmt_number_02;
+		    return _fmt_number_02( @_[ 0, 1 ],
+			__week_of_year(
+			    $_[0]->__fmt_shire_month(),
+			    $_[0]->__fmt_shire_day(),
+			),
+		    );
 		},
 	u	=> sub { $_[0]->__fmt_shire_day_of_week() },
 #	V	Same as U by definition of Shire calendar
@@ -260,8 +293,8 @@ sub _fmt_cond {
 	x	=> sub { __format( $_[0], '%{{%e %b %Y||%Ee %Y}}' ) }, 
 	Y	=> sub { $_[0]->__fmt_shire_year() },
 	y	=> sub {
-		    $_[2] = $_[0]->__fmt_shire_year() % 100;
-		    goto &_fmt_number_02;
+		    return _fmt_number_02( @_[ 0, 1 ],
+			$_[0]->__fmt_shire_year() % 100 );
 		},
 	Z	=> sub { $_[0]->__fmt_shire_zone_name() },
 	z	=> sub { _fmt_offset( $_[0]->__fmt_shire_zone_offset() ) },
@@ -832,58 +865,60 @@ sub _make_date_object {
     my $ref = ref $date
 	or Carp::croak( FORMAT_DATE_ERROR );
 
-    if ( HASH_REF eq $ref ) {
-	my %hash = %{ $date };
-	$hash{day} ||= 1;
-	$hash{month} ||= $hash{day} < 7 ? 0 : 1;
-	$hash{$_} ||= 0 for qw{
-	    hour minute second nanosecond epoch
-	};
-	defined $hash{zone_name}
-	    or $hash{zone_name} = '';
-	$date = bless \%hash, DATE_CLASS;
-    }
+    HASH_REF eq $ref
+	or return __valid_date_class( $date );
 
-    {
-	my @missing = __valid_date_class( $date );
-	local $" = ', ';
-	@missing
-	    and Carp::croak(
-	    "A @{[ ref $date ]} is not a valid date object, it lacks: @missing" );
-    }
-
-    return $date;
+    my %hash = %{ $date };
+    $hash{day} ||= 1;
+    $hash{month} ||= $hash{day} < 7 ? 0 : 1;
+    $hash{$_} ||= 0 for qw{
+	hour minute second nanosecond epoch
+    };
+    defined $hash{zone_name}
+	or $hash{zone_name} = '';
+    return bless \%hash, DATE_CLASS;
 }
 
-sub __valid_date_class {
-    my ( $obj ) = @_;
-    my $package = ref $obj || $obj;
-    unless ( ref $obj ) {
-	( my $fn = $package ) =~ s{ :: }{/}smxg;
-	$fn .= '.pm';
-	$INC{$fn}
-	    or require $fn;
+{
+    my %checked;
+
+    sub __valid_date_class {
+	my ( $obj ) = @_;
+	my $pkg = ref $obj || $obj;
+
+	local $" = ', ';
+	@{ $checked{$pkg} ||= do {
+	    unless ( ref $obj ) {
+		( my $fn = $pkg ) =~ s{ :: }{/}smxg;
+		$fn .= '.pm';
+		$INC{$fn}
+		    or require $fn;
+	    }
+	    my @missing;
+	    foreach my $method ( qw{
+		__fmt_shire_year
+		__fmt_shire_month
+		__fmt_shire_day
+		__fmt_shire_hour
+		__fmt_shire_minute
+		__fmt_shire_second
+		__fmt_shire_day_of_week
+		__fmt_shire_nanosecond
+		__fmt_shire_epoch
+		__fmt_shire_zone_offset
+		__fmt_shire_zone_name
+		__fmt_shire_accented
+		__fmt_shire_traditional
+	    } ) {
+		$pkg->can( $method )
+		    or push @missing, $method;
+	    }
+	    \@missing;
+	} }
+	    and Carp::croak(
+	    "$pkg lacks methods: @{ $checked{$pkg} }" );
+	return $obj;
     }
-    my @missing;
-    foreach my $method ( qw{
-	__fmt_shire_year
-	__fmt_shire_month
-	__fmt_shire_day
-	__fmt_shire_hour
-	__fmt_shire_minute
-	__fmt_shire_second
-	__fmt_shire_day_of_week
-	__fmt_shire_nanosecond
-	__fmt_shire_epoch
-	__fmt_shire_zone_offset
-	__fmt_shire_zone_name
-	__fmt_shire_accented
-	__fmt_shire_traditional
-    } ) {
-	$package->can( $method )
-	    or push @missing, $method;
-    }
-    return @missing;
 }
 
 # The arguments are multiple array references. The hash is set up so
@@ -1714,22 +1749,26 @@ returned. Otherwise, C<undef> is returned.
 
 =head2 __valid_date_class
 
+ eval {
+     __valid_date_class( $class );
+     print "$class OK\n";
+     1;
+ } or print $@;
+
 This subroutine takes as its argument either an object or a class name.
-If a class name, it is loaded if possible, and if it has not already
-been loaded. Then the class is checked to see if it has all the methods
-required of a date object passed to L<__format()|/__format>.
+If a class name, it is loaded if needed and possible. Then the class is
+checked to see if it has all the methods required of a date object
+passed to L<__format()|/__format>. If any missing methods are found an
+exception is thrown naming the missing methods; otherwise it returns its
+argument.
 
-In list context it returns the names of any missing methods. In scalar
-context it returns the number of missing methods. So a return of an
-empty list (or zero in scalar context) means the class can probably be
-passed as a date to L<__format()|/__format()>. The weasel word
-'probably' is because the check is on method name only; we have no idea
-whether the semantics of the methods found are what we would like them
-to be.
+If you intend to use a class that autoloads requisite methods, that
+class will need to properly override L<can()|UNIVERSAL/can>, or provide
+forward references to autoloaded methods.
 
-This subroutine is exposed for troubleshooting purposes, though it is
-hoped the exception thrown if this check fails will be sufficiently
-explicit. See also F<tools/valid-date-class>.
+This subroutine is used internally to validate the date argument to
+L<__format()|/__format>. It is exposed for troubleshooting purposes.
+See also F<tools/valid-date-class>.
 
 =head2 __weekday_name
 
